@@ -1,13 +1,21 @@
 from django.conf import settings
 from django.db import models, connection, transaction
-from tenant_schemas.postgresql_backend.base import _check_identifier
+from django.db.models.signals import post_delete
+from django.dispatch import receiver
 from django.core.management import call_command
+from django.utils.functional import lazy
+from tenant_schemas.postgresql_backend.base import _check_identifier
 from tenant_schemas.signals import post_schema_sync
-from tenant_schemas.utils import django_is_in_test_mode, schema_exists
-from .utils import get_public_schema_name
+from tenant_schemas.utils import django_is_in_test_mode, schema_exists, get_tenant_model
+from tenant_schemas.utils import get_public_schema_name
 
 
 class TenantMixin(models.Model):
+    auto_drop_schema = False   # USE THIS WITH CAUTION!
+                               # set this flag to true on a parent class if
+                               # you want the schema to be automatically
+                               # removed after tenant remove.
+
     auto_create_schema = True  # set this flag to false on a parent class if
                                # you dont want the schema to be automatically
                                # created upon save.
@@ -19,11 +27,15 @@ class TenantMixin(models.Model):
         abstract = True
 
     def save(self, verbosity=1, *args, **kwargs):
-        if connection.get_schema() != get_public_schema_name():
-            raise Exception("Can't update tenant outside the public schema. Current schema is %s."
+        is_new = self.pk is None
+
+        if is_new and connection.get_schema() != get_public_schema_name():
+            raise Exception("Can't create tenant outside the public schema. Current schema is %s."
+                            % connection.get_schema())
+        elif not is_new and connection.get_schema() not in (self.schema_name, get_public_schema_name()):
+            raise Exception("Can't update tenant outside it's own schema or the public schema. Current schema is %s."
                             % connection.get_schema())
 
-        is_new = self.pk is None
         super(TenantMixin, self).save(*args, **kwargs)
 
         if is_new and self.auto_create_schema:
@@ -32,6 +44,13 @@ class TenantMixin(models.Model):
 
         transaction.commit_unless_managed()
 
+    def delete(self, *args, **kwargs):
+        if connection.get_schema() not in (self.schema_name, get_public_schema_name()):
+            raise Exception("Can't delete tenant outside it's own schema or the public schema. Current schema is %s."
+                            % connection.get_schema())
+
+        super(TenantMixin, self).delete(*args, **kwargs)
+
     def create_schema(self, check_if_exists=False, sync_schema=True, verbosity=1):
         """
         Creates the schema 'schema_name' for this tenant. Optionally checks if the schema
@@ -63,4 +82,17 @@ class TenantMixin(models.Model):
             if 'south' in settings.INSTALLED_APPS and not django_is_in_test_mode():
                 call_command('migrate_schemas', fake=True, schema_name=self.schema_name, verbosity=verbosity)
 
+        connection.set_schema_to_public()
+
         return True
+
+@receiver(post_delete, sender=lazy(get_tenant_model))
+def drop_schema(sender, instance, **kwargs):
+    """
+    Called in post_delete signal.
+    Drops the schema related to the tenant instance. Just drop the schema if the parent
+    class model has the attribute auto_drop_schema set to True.
+    """
+    if schema_exists(instance.schema_name) and instance.auto_drop_schema:
+        cursor = connection.cursor()
+        cursor.execute('DROP SCHEMA %s CASCADE' % instance.schema_name)
